@@ -6,6 +6,7 @@ from dataclasses import dataclass
 from typing import Any, List, Mapping
 
 import dpath
+
 from airbyte_cdk.sources.declarative.auth.declarative_authenticator import DeclarativeAuthenticator
 
 
@@ -27,9 +28,16 @@ class SelectiveAuthenticator(DeclarativeAuthenticator):
         **kwargs: Any,
     ) -> DeclarativeAuthenticator:
         try:
-            selected_key = str(dpath.get(config, authenticator_selection_path))
+            selected_key = str(
+                dpath.get(
+                    config,  # type: ignore[arg-type]  # Dpath wants mutable mapping but doesn't need it.
+                    authenticator_selection_path,
+                )
+            )
         except KeyError as err:
-            raise ValueError("The path from `authenticator_selection_path` is not found in the config.") from err
+            raise ValueError(
+                "The path from `authenticator_selection_path` is not found in the config."
+            ) from err
 
         try:
             return authenticators[selected_key]
